#ifndef __XLIC_BASE64_HPP__
#define __XLIC_BASE64_HPP__

#include <vector>
#include <string>
#include <cstddef>
#include <cstdint>
#include <cstring>
#include "config.h"

struct base64 {

	static std::vector<uint8_t> decode(const char * input);
	static std::vector<uint8_t> decode(
		const void * input,
		const size_t length);
	static std::vector<uint8_t> decode(const std::string & input);
	static size_t decode(
		void * output,
		const void * input,
		const size_t length) throw();

	static std::string encode(const char * input);
	static std::string encode(
		const void * input,
		const size_t length);
	static std::string encode(const std::string & input);
	template<class T> static std::string encode(const std::vector<T> & input);
	static size_t encode(
		void * output,
		const void * input,
		const size_t length) throw();

	static bool valid(const char * input) throw();
	static bool valid(const std::string & input) throw();
	static bool valid(
		const void * input,
		const size_t length) throw();

	static size_t decoded_length(
		const void * input,
		const size_t length) throw();

	static size_t encoded_length(const size_t length) throw();
};

inline size_t base64::encoded_length(const size_t length) throw() {
	return (((length - 1) + (3 - ((length - 1) % 3))) / 3) * 4;
}

inline std::vector<uint8_t> base64::decode(const char * input) {
	return decode(input, strlen(input));
}

inline std::vector<uint8_t> base64::decode(
	const void * input,
	const size_t length
) {
	std::vector<uint8_t> output((length / 4) * 3);
	output.resize(decode(output.data(), input, length));
	return output;
}

inline std::vector<uint8_t> base64::decode(const std::string & input) {
	return decode(input.data(), sizeof(std::string::value_type) * input.length());
}

inline std::string base64::encode(const char * input) {
	return encode(input, strlen(input));
}

inline std::string base64::encode(
	const void * input,
	const size_t length
) {
	std::string output(encoded_length(length), 0);
	output.resize(encode(&output[0], input, length));
	return output;
}

inline std::string base64::encode(const std::string & input) {
	return encode(input.data(), sizeof(std::string::value_type) * input.length());
}

template<class T>
inline std::string base64::encode(const std::vector<T> & input) {
	return encode(input.data(), sizeof(T) * input.size());
}

inline bool base64::valid(const char * input) throw() {
	return valid(input, strlen(input));
}

inline bool base64::valid(const std::string & input) throw() {
	return valid(input.data(), sizeof(std::string::value_type) * input.length());
}

#endif
