#ifndef __EVP_BIO_HPP__
#define __EVP_BIO_HPP__

#include <openssl/bio.h>
#include "config.h"

class evp_bio_t {
public:

	// This should be const but OpenSSL won't let us
	evp_bio_t(BIO_METHOD * method);
	evp_bio_t(evp_bio_t && bio);
	evp_bio_t(const evp_bio_t & bio) = delete;
	~evp_bio_t();

	const BIO * data() const throw();

	void write(const std::string & string);

	evp_bio_t & operator=(evp_bio_t && bio);
	evp_bio_t & operator=(const evp_bio_t & bio) = delete;

private:

	BIO * m_data;
};

inline evp_bio_t::evp_bio_t(BIO_METHOD * method)
: m_data(BIO_new(method)) {
	if (m_data == nullptr) { throw std::bad_alloc(); }
}

inline evp_bio_t::~evp_bio_t() {
	BIO_free_all(m_data);
}

inline const BIO * evp_bio_t::data() const throw() {
	return m_data;
}

inline void evp_bio_t::write(const std::string & string) {
	if (static_cast<size_t>(BIO_write(m_data, string.data(), string.length())) != string.length()) {
		throw std::bad_alloc();
	}
}

#endif
