#include "evp_context.hpp"

evp_context_t::evp_context_t(
	const EVP_MD * algo,
	const evp_public_key_t & key
) : m_context(EVP_MD_CTX_create()) {
	if (m_context == nullptr) {
		throw std::bad_alloc();
	}
	// OpenSSL doesn't understand const
	if (EVP_DigestVerifyInit(
		m_context,
		nullptr,
		algo,
		nullptr,
		const_cast<EVP_PKEY *>(key.data())
	) != 1) {
		throw std::bad_alloc();
	}
}

bool evp_context_t::verify(
	const void * input,
	const size_t length,
	const signature_t & signature
) {
	if (EVP_DigestVerifyUpdate(m_context, input, length) != 1) {
		throw std::bad_alloc();
	}
	return (EVP_DigestVerifyFinal(
		m_context,
		// OpenSSL doesn't understand const nor allows varying inputs
		reinterpret_cast<uint8_t *>(const_cast<signature_t *>(&signature)),
		sizeof(xlic_signature)
	) == 1);
}
