#ifndef __EVP_CONTEXT_HPP__
#define __EVP_CONTEXT_HPP__

#include <openssl/evp.h>
#include "evp_public_key.hpp"
#include "signature.hpp"
#include "config.h"

class evp_context_t {
public:

	evp_context_t(
		const EVP_MD * algo,
		const evp_public_key_t & key);
	evp_context_t(const evp_context_t & context) = delete;
	evp_context_t(evp_context_t &&);

	~evp_context_t();

	bool verify(
		const void * input,
		const size_t length,
		const signature_t & signature);

	evp_context_t & operator=(const evp_context_t & context) = delete;
	evp_context_t & operator=(evp_context_t &&);

private:

	EVP_MD_CTX * m_context;
};

inline evp_context_t::~evp_context_t() {
	EVP_MD_CTX_destroy(m_context);
}

#endif
