#ifndef __EVP_PUBLIC_KEY_HPP__
#define __EVP_PUBLIC_KEY_HPP__

#include <openssl/evp.h>
#include <string>
#include "config.h"

class evp_public_key_t {
public:

	evp_public_key_t(EVP_PKEY * key);
	evp_public_key_t(const evp_public_key_t & key) = delete;
	evp_public_key_t(evp_public_key_t && key);

	~evp_public_key_t();

	const EVP_PKEY * data() const throw();

	static evp_public_key_t from_pem(const std::string & pem);

	evp_public_key_t & operator=(const evp_public_key_t & key) = delete;
	evp_public_key_t & operator=(evp_public_key_t && key);

private:

	EVP_PKEY * m_key;
};

inline evp_public_key_t::evp_public_key_t(EVP_PKEY * key)
: m_key(key) {
	if (key == nullptr) { throw std::bad_alloc(); }
}

inline evp_public_key_t::~evp_public_key_t() {
	EVP_PKEY_free(m_key);
}

inline const EVP_PKEY * evp_public_key_t::data() const throw() {
	return m_key;
}

#endif
