#include "hex.hpp"
#include "globals.hpp"

size_t hex::decode(
	void * output,
	const void * input,
	const size_t length
) throw() {
	const uint8_t * in = (const uint8_t *) input;
	uint8_t * out = (uint8_t *) output;
	for (size_t i = 0, end = decoded_length(length); i < end; i++) {
		uint8_t byte = hex2bin_map[*in++] << 4;
		*out++ = byte | hex2bin_map[*in++];
	}
	return decoded_length(length);
}

size_t hex::encode(
	void * output,
	const void * input,
	const size_t length
) throw() {
	const uint8_t * in = (const uint8_t *) input;
	char * out = (char *) output;
	for (size_t i = 0; i < length; i++) {
		const uint8_t b = *in++;
		*out++ = bin2hex_map[b >> 4];
		*out++ = bin2hex_map[b & 0xf];
	}
	return encoded_length(length);
}

bool hex::valid(
	const void * input,
	const size_t length
) throw() {
	const char * in = static_cast<const char *>(input);
	for (size_t i = 0; i < length; i++) {
		const char c = *in++;
		if (!(
			(c >= '0' && c <= '9') ||
			(c >= 'a' && c <= 'f') ||
			(c >= 'A' && c <= 'F')
		)) {
			return false;
		}
	}
	return true;
}
