#ifndef __LICENSE_FILE_HPP__
#define __LICENSE_FILE_HPP__

#include <list>
#include <memory>
#include <string>
#include "license.hpp"
#include "config.h"

class license_file {
public:

	license_file(const std::string & filename);

	~license_file() = default;

	void add(const license_t & license);

	void count() const throw();

	xlic_license_list * export_list() const;

	const license_t * find(const std::string & product_name) const;

	const license_t * find(const license_key_t & license_key) const throw();

	const public_key_t & public_key() const throw();
	void public_key(const public_key_t & key) throw();

	bool remove(
		const std::string & url,
		const license_key_t & license_key);

	void save(const std::string & filename) const;

	void validate(
		const std::string & url,
		const std::string & alternative_serial_number,
		const std::string & filename);

private:

	public_key_t m_public_key;
	std::list<license_t> m_licenses;
};

inline const public_key_t & license_file::public_key() const throw() {
	return m_public_key;
}

inline void license_file::public_key(const public_key_t & key) throw() {
	m_public_key = key;
}

namespace std {

	/*! Helper which automatically cleans up xlic_license_list pointers on scope exit. */
	template<>
	class default_delete<xlic_license_list> {
	public:

		inline void operator()(xlic_license_list * list) const {
			delete[] reinterpret_cast<char *>(list);
		}
	};

	/*! Helper which automatically cleans up xlic_license pointers on scope exit. */
	template<>
	class default_delete<xlic_license> {
	public:

		inline void operator()(xlic_license * license) const {
			delete[] reinterpret_cast<char *>(license);
		}
	};
}

#endif
