#ifndef __LOCKFILE_HPP__
#define __LOCKFILE_HPP__

#include <sys/file.h>
#include <unistd.h>
#include "config.h"

class lockfile {
public:

	lockfile(const char * filename) throw();

	~lockfile() throw();

	bool acquire() throw();

	void release() throw();

private:

	int handle;

	lockfile(const lockfile &) = delete;
	lockfile(lockfile &&) = delete;
	lockfile & operator=(const lockfile &) = delete;
	lockfile & operator=(lockfile &&) = delete;
};

inline lockfile::lockfile(const char * filename) throw()
: handle(open(filename, O_RDONLY | O_CREAT, 0666)) {}

inline lockfile::~lockfile() throw() {
	if (handle != -1) { close(handle); }
}

inline bool lockfile::acquire() throw() {
	if (handle != -1) {
		return (flock(handle, LOCK_NB | LOCK_EX) == 0);
	}
	return false;
}

inline void lockfile::release() throw() {
	if (handle != -1) {
		flock(handle, LOCK_UN);
		close(handle);
	}
}

#endif
