#ifndef __PUBLIC_KEY_HPP__
#define __PUBLIC_KEY_HPP__

#include <cstdint>
#include <cstring>
#include <xorcom/licensing.h>
#include "hex.hpp"
#include "base64.hpp"
#include "config.h"

class public_key_t : public xlic_public_key {
public:

	static const public_key_t invalid;

	public_key_t();
	explicit public_key_t(const xlic_public_key & input) throw();

	static public_key_t from_hex(const char * input) throw();
	static public_key_t from_hex(
		const void * input,
		const size_t length) throw();

	std::string to_string() const;

	std::string to_base64() const;

	std::string to_hex() const;

	std::string to_pem() const;

	public_key_t & operator=(const xlic_public_key & input) throw();

	bool operator==(const public_key_t & key) throw();
	bool operator==(const xlic_public_key & key) throw();

	bool operator!=(const public_key_t & key) throw();
	bool operator!=(const xlic_public_key & key) throw();
};

inline public_key_t::public_key_t()
: xlic_public_key(invalid) {}

inline public_key_t::public_key_t(const xlic_public_key & input) throw()
: xlic_public_key(input) {}

inline public_key_t public_key_t::from_hex(const char * input) throw() {
	return from_hex(input, strlen(input));
}

inline public_key_t public_key_t::from_hex(
	const void * input,
	const size_t length
) throw() {
	if (hex::decoded_length(length) == sizeof(xlic_public_key) && hex::valid(input, length)) {
		public_key_t tmp;
		hex::decode(tmp.data, input, length);
		return tmp;
	}
	return invalid;
}

inline std::string public_key_t::to_base64() const {
	return base64::encode(data, sizeof(xlic_public_key));
}

inline std::string public_key_t::to_hex() const {
	return hex::encode(data, sizeof(xlic_public_key));
}

inline bool public_key_t::operator==(const public_key_t & key) throw() {
	return (memcmp(data, key.data, sizeof(xlic_public_key)) == 0);
}

inline bool public_key_t::operator==(const xlic_public_key & key) throw() {
	return (memcmp(data, key.data, sizeof(xlic_public_key)) == 0);
}

inline bool public_key_t::operator!=(const public_key_t & key) throw() {
	return (memcmp(data, key.data, sizeof(xlic_public_key)) != 0);
}

inline bool public_key_t::operator!=(const xlic_public_key & key) throw() {
	return (memcmp(data, key.data, sizeof(xlic_public_key)) != 0);
}

#endif
