#include <string>
#include <system_error>
#include <xorcom/licensing.h>
#include "lockfile.hpp"
#include "license.hpp"
#include "license_file.hpp"
#include "globals.hpp"
#include "string_ref.hpp"
#include "util.hpp"

extern "C" int xlic_register_license(
	const char * filename,
	const char * url,
	const char * license_key,
	const char * serial_number
) {
	try {
		license_t license;
		license.key = license_key_t::from_base64(license_key);
		if (license.key == license_key_t::invalid) {
			throw std::logic_error("Invalid license key provided");
		}
		lockfile lock(XLIC_LOCKFILE);
		if (lock.acquire()) {
			license_file license_bin(filename);
			if (license_bin.find(license.key)) {
				throw std::logic_error("The license key is already registered to this system");
			}
			public_key_t public_key;
			std::string complete_url(string_ref(url) + "register");
			if (license.validate(complete_url.data(), public_key, serial_number)) {
				license_bin.public_key(public_key);
				license_bin.add(license);
				license_bin.save(filename);
				return EXIT_SUCCESS;
			} else {
				fprintf(stderr, "Registration rejected by server\n");
			}
		} else {
			fprintf(stderr, "License file is locked, cannot perform operation at this time\n");
			return EBUSY;
		}
	} catch (std::exception & e) {
		fprintf(stderr, "Error: %s\n", e.what());
	}
	return EINVAL;
}
