#include <system_error>
#include "license_file.hpp"
#include "lockfile.hpp"
#include "base64.hpp"
#include "globals.hpp"
#include "string_ref.hpp"

extern "C" int xlic_update_licenses(
	const char * filename,
	const char * url
) {
	try {
		lockfile lock(XLIC_LOCKFILE);
		if (lock.acquire()) {
			license_file license_bin(filename);
			license_bin.validate((string_ref(url) + "register").data(), "", filename);
			return EXIT_SUCCESS;
		} else {
			fprintf(stderr, "License file is locked, cannot perform operation at this time\n");
			return EBUSY;
		}
	} catch (std::system_error & e) {
		fprintf(stderr, "Error: %s\n", e.what());
		return e.code().value();
	} catch (std::exception & e) {
		fprintf(stderr, "Error: %s\n", e.what());
	}
	return EINVAL;
}
