#ifndef __URL_HPP__
#define __URL_HPP__

#include <vector>
#include <string>
#include <cstddef>
#include <cstdint>
#include <cstring>
#include "config.h"

class url {
public:

	static std::vector<uint8_t> decode(const char * input);
	static std::vector<uint8_t> decode(
		const void * input,
		const size_t length);
	static std::vector<uint8_t> decode(const std::string & input);
	static size_t decode(
		void * output,
		const void * input,
		const size_t length) throw();

	static std::string encode(const char * input);
	static std::string encode(
		const void * input,
		const size_t length);
	static std::string encode(const std::string & input);
	static size_t encode(
		void * output,
		const void * input,
		const size_t length) throw();

	static bool valid(const char * input) throw();
	static bool valid(const std::string & input) throw();
	static bool valid(
		const void * input,
		const size_t length) throw();

	static size_t decoded_length(
		const void * input,
		const size_t length) throw();

	static size_t encoded_length(
		const void * input,
		const size_t length) throw();
};

inline std::vector<uint8_t> url::decode(const char * input) {
	return decode(input, strlen(input));
}

inline std::vector<uint8_t> url::decode(
	const void * input,
	const size_t length
) {
	std::vector<uint8_t> output(decoded_length(input, length));
	decode(output.data(), input, length);
	return output;
}

inline std::vector<uint8_t> url::decode(const std::string & input) {
	return decode(input.data(), sizeof(std::string::value_type) * input.length());
}

inline std::string url::encode(const char * input) {
	return encode(input, strlen(input));
}
inline std::string url::encode(const std::string & input) {
	return encode(input.data(), sizeof(std::string::value_type) * input.length());
}

inline std::string url::encode(
	const void * input,
	const size_t length
) {
	std::string output(encoded_length(input, length), 0);
	encode(&output[0], input, length);
	return output;
}

inline bool url::valid(const char * input) throw() {
	return valid(input, strlen(input));
}

inline bool url::valid(const std::string & input) throw() {
	return valid(input.data(), sizeof(std::string::value_type) * input.length());
}

#endif
